// $Id$

/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef _KBIBTEXPART_H_
#define _KBIBTEXPART_H_

#include <kparts/part.h>
#include <kparts/factory.h>

class QWidget;
class QPainter;
class QListViewItem;
class QProgressDialog;
class KURL;
class KListView;
class KRecentFilesAction;
class KAction;
class KBibTeXFile;
class KBibTeXListView;
class BibTeXElement;

/**
 * This is a "Part".  It that does all the real work in a KPart
 * application.
 *
 * @short Main Part
 * @author Thomas Fischer <fischer@unix-ag.uni-kl.de>
 * @version 20050316
 */
class KBibTeXPart : public KParts::ReadWritePart
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                 QObject *parent, const char *name );

    /**
     * Destructor
     */
    virtual ~KBibTeXPart();

    /**
     * This is a virtual function inherited from KParts::ReadWritePart.
     * A shell will use this to inform this Part if it should act
     * read-only
     */
    virtual void setReadWrite( bool rw );

    /**
     * Reimplemented to disable and enable Save action
     */
    virtual void setModified( bool modified );
    void readSettings();
    void writeSettings();

    void insertText( QString &text );

protected:
    /**
     * This must be implemented by each part
     */
    virtual bool openFile();

    /**
     * This must be implemented by each read-write part
     */
    virtual bool saveFile();

private slots:
    void fileOpen();
    void fileSaveAs();
    void newElement();
    bool editElement( QListViewItem* item = NULL );
    void deleteElement();
    void showProgress( int current, int total );
    void cutElement();
    void copyElement();
    void pasteElement();
    void showBibtexListContextMenu( KListView *l, QListViewItem *i, const QPoint &p );
    void slotSelectionChanged();
    void slotFind();
    void slotFindPrev();
    void slotFindNext();
    void slotSearchGoogle();
    void slotSearchCiteseer();

private:
    KBibTeXListView *m_bibtexlist;
    KBibTeXFile *m_bibtexfile;
    int newElementCount;
    QProgressDialog *progressDialog;
    KRecentFilesAction *recent;
    KAction *actionElement;
    KAction *actionEditElement;
    KAction *actionDeleteElement;
    KAction *actionEditCut;
    KAction *actionEditCopy;
    KAction *actionEditPaste;
    KAction *actionEditFind;
    KAction *actionEditFindNext;
    KAction *actionEditFindPrev;
    KAction *actionEditSearchGoogle;
    KAction *actionEditSearchCiteseer;
    QString findPattern;
    bool findCaseSensitive;

    void setupGUI();
    void setupActions();
    void updateMenu();
    void find( QListViewItem *startAt, bool downwards = true );
    void searchInternet( BibTeXElement *element, int engine );

signals:
    void progressCancel();
};

class KInstance;
class KAboutData;

class KBibTeXPartFactory : public KParts::Factory
{
    Q_OBJECT
public:
    KBibTeXPartFactory();
    virtual ~KBibTeXPartFactory();
    virtual KParts::Part* createPartObject( QWidget *parentWidget, const char *widgetName,
                                            QObject *parent, const char *name,
                                            const char *classname, const QStringList &args );
    static KInstance* instance();

private:
    static KInstance* s_instance;
    static KAboutData* s_about;
};

#endif // _KBIBTEXPART_H_
